-- HepHUD v0.4 20140503

HepHUD = {}
HepHUD.cName = 'HepHUD'
HepHUD.nVersion = 0.4
HepHUD.nSpacing = 40
HepHUD.nGap = 350
HepHUD.nTop = 0
HepHUD.oAlpha = {nNormal = 0.3, nUsed = 0.6}
HepHUD.oBarSize = {nWidth = 110, nHeight = 512}
HepHUD.lHidden = false
HepHUD.nLeft = 0
HepHUD.nRight = 0
HepHUD.aIcons = {
	['veteran'] = {0.8, 0},
	['dominion'] = {0.9, 0},
	['covenant'] = {0.8, 0.1},
	['ebonheart'] = {0.9, 0.1},
	['health'] = {0.8, 0.2},
	['magicka'] = {0.9, 0.2},
	['stamina'] = {0.8, 0.3},
	['armor'] = {0.9, 0.3},
	['power'] = {0.8, 0.4},
	['deadly'] = {0.9, 0.4},
	['hard'] = {0.8, 0.5},
	['normal'] = {0.9, 0.5}}
HepHUD.aVisuals = {
	[STAT_ARMOR_RATING] = 'armor',
	[STAT_ATTACK_POWER] = 'power',
	[STAT_HEALTH_MAX] = 'health',
	[STAT_MAGICKA_MAX] = 'magicka',
	[STAT_POWER] = 'power',
	[STAT_SPELL_POWER] = 'power',
	[STAT_STAMINA_MAX] = 'stamina',
	[STAT_WEAPON_POWER] = 'power'}

local function chain(o)
	
	-- Function to easily create properties on an object.
	
	local t = {}
	setmetatable(t, {__index = function(self, f)
		if f == '__END' then return o end
		return function(self, ...)
			assert(o[f], f .. ' missing in object')
			o[f](o, ...)
			return self
		end
	end})
	return t
	
end

local function createBar(oParams)
	
	-- Creates a single bar. Default parameters.
	
	if oParams == nil then oParams = {} end
	if oParams.cSide == nil then oParams.cSide = 'left' end
	if oParams.aColors == nil then oParams.aColors = {192, 0, 0} end
	if oParams.cTrack == nil then oParams.cTrack = 'Player Health' end
	
	-- Set up some basic values.
	
	local cTexture = HepHUD.cName .. '/bars.dds'
	local nWidth = 0.2
	local nGradient = 32
	local nBar, nBG = nWidth * 3, nWidth
	local nX = HepHUD.nGap
	
	-- Figure out its position.
	
	local nPosition, nModify = HepHUD.nRight, 1
	
	-- If the bar is on the left side, make its position negative.
	-- Also, shift its texture position.
	
	if oParams.cSide == 'left' then
		nPosition, nModify = HepHUD.nLeft, -1
		nBar, nBG = nWidth * 2, 0
		HepHUD.nLeft = HepHUD.nLeft + 1
	else
		HepHUD.nRight = HepHUD.nRight + 1
	end
	
	nX = (nX + nPosition * HepHUD.nSpacing) * nModify
	
	-- Create the total frame.
	
	local oReturn = chain(WINDOW_MANAGER:CreateTopLevelWindow(nil))
		:SetDimensions(HepHUD.oBarSize.nWidth, HepHUD.oBarSize.nHeight)
		:SetAnchor(CENTER, GuiRoot, CENTER, nX, HepHUD.nTop)
		:SetMouseEnabled(false)
	.__END
	
	-- The background.
	
	oReturn.oBG = chain(WINDOW_MANAGER:CreateControl(nil, oReturn, CT_TEXTURE))
		:SetMouseEnabled(false)
		:SetTexture(cTexture)
		:SetTextureCoords(nBG, nBG + nWidth, 0, 1)
		:SetDimensions(HepHUD.oBarSize.nWidth, HepHUD.oBarSize.nHeight)
		:SetGradientColors(ORIENTATION_VERTICAL,
			oParams.aColors[1] / 255,
			oParams.aColors[2] / 255,
			oParams.aColors[3] / 255, 1,
			oParams.aColors[1] / 255,
			oParams.aColors[2] / 255,
			oParams.aColors[3] / 255, 1)
		:SetAnchor(BOTTOMLEFT, oReturn, BOTTOMLEFT, 0, 0)
		:SetAlpha(HepHUD.oAlpha.nNormal)
	.__END
	
	-- The bar itself.
	
	oReturn.oBar = chain(WINDOW_MANAGER:CreateControl(nil, oReturn, CT_TEXTURE))
		:SetMouseEnabled(false)
		:SetTexture(cTexture)
		:SetTextureCoords(nBar, nBar + nWidth, 0, 1)
		:SetDimensions(HepHUD.oBarSize.nWidth, HepHUD.oBarSize.nHeight)
		:SetGradientColors(ORIENTATION_VERTICAL,
			math.max(oParams.aColors[1] - nGradient, 0) / 255,
			math.max(oParams.aColors[2] - nGradient, 0) / 255,
			math.max(oParams.aColors[3] - nGradient, 0) / 255, 1,
			math.min(oParams.aColors[1] + nGradient, 255) / 255,
			math.min(oParams.aColors[2] + nGradient, 255) / 255,
			math.min(oParams.aColors[3] + nGradient, 255) / 255, 1)
		:SetAnchor(BOTTOMLEFT, oReturn, BOTTOMLEFT, 0, 0)
		:SetAlpha(HepHUD.oAlpha.nNormal)
	.__END
	
	-- Save the texture coordinates, colors, etc for later.
	
	oReturn.aTextureCoords = {nBar, nBar + nWidth}
	oReturn.aColors = oParams.aColors
	oReturn.cTrack = oParams.cTrack
	oReturn.lRegen = false
	oReturn.lDegen = false
	
	-- Return the control.
	
	return oReturn
	
end

local function createInfoFrame(oParams)
	
	-- Create a frame for the player and target level and rank.
	
	if oParams == nil then oParams = {} end
	if oParams.cSide == nil then oParams.cSide = 'left' end
	if oParams.oAlign == nil then oParams.oAlign = HepHUD.oTarget end
	
	-- Figure out its position.
	
	local cTexture = HepHUD.cName .. '/bars.dds'
	local nSize, nAnchor1, nAnchor2, nLevel, nRank = 51, TOPLEFT, BOTTOMLEFT, 0, 1
	local aLevelCoords = HepHUD.getIconCoords('veteran')
	local aRankCoords = HepHUD.getIconCoords('dominion')
	
	if oParams.cSide == 'left' then nAnchor1, nAnchor2, nLevel, nRank = TOPRIGHT, BOTTOMRIGHT, 1, 0 end
	
	-- Create the frames, then the controls inside of them.
	
	local oReturn = chain(WINDOW_MANAGER:CreateTopLevelWindow(nil))
		:SetDimensions(nSize * 2, nSize)
		:SetAnchor(nAnchor2, oParams.oAlign, nAnchor1, 0, -10)
		:SetMouseEnabled(false)
	.__END
	
	-- The Level.
	
	oReturn.oLevelBG = chain(WINDOW_MANAGER:CreateControl(nil, oReturn, CT_TEXTURE))
		:SetMouseEnabled(false)
		:SetTexture(cTexture)
		:SetTextureCoords(aLevelCoords[1], aLevelCoords[1] + 0.1, aLevelCoords[2], aLevelCoords[2] + 0.1)
		:SetDimensions(nSize, nSize)
		:SetColor(1, 1, 1, 1)
		:SetAnchor(TOPLEFT, oReturn, TOPLEFT, nLevel * nSize, 0)
		:SetAlpha(0.75)
	.__END
	
	oReturn.oLevel = chain(WINDOW_MANAGER:CreateControl(nil, oReturn, CT_LABEL))
		:SetMouseEnabled(false)
		:SetDimensions(nSize, nSize)
		:SetColor(1, 1, 1, 1)
		:SetAnchor(TOPLEFT, oReturn, TOPLEFT, nLevel * nSize, nSize / 2 - 12)
		:SetFont('ZoFontCallout')
		:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
		:SetVerticalAlignment(TEXT_ALIGN_TOP)
		:SetAlpha(1)
		:SetText('00')
	.__END
	
	-- The Rank.
	
	oReturn.oRankBG = chain(WINDOW_MANAGER:CreateControl(nil, oReturn, CT_TEXTURE))
		:SetMouseEnabled(false)
		:SetTexture(cTexture)
		:SetTextureCoords(aRankCoords[1], aRankCoords[1] + 0.1, aRankCoords[2], aRankCoords[2] + 0.1)
		:SetDimensions(nSize, nSize)
		:SetColor(1, 1, 1, 1)
		:SetAnchor(TOPLEFT, oReturn, TOPLEFT, nRank * nSize - 1, 0)
		:SetAlpha(0.75)
	.__END
	
	oReturn.oRank = chain(WINDOW_MANAGER:CreateControl(nil, oReturn, CT_LABEL))
		:SetMouseEnabled(false)
		:SetDimensions(nSize, nSize)
		:SetColor(1, 1, 1, 1)
		:SetAnchor(TOPLEFT, oReturn, TOPLEFT, nRank * nSize - 1, nSize / 2 - 12)
		:SetFont('ZoFontCallout')
		:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
		:SetVerticalAlignment(TEXT_ALIGN_TOP)
		:SetAlpha(1)
		:SetText('00')
	.__END
	
	return oReturn
	
end

local function createBuffFrame(oParams)
	
	-- Create a frame for the player and target buffs and debuffs.
	
	if oParams == nil then oParams = {} end
	if oParams.cSide == nil then oParams.cSide = 'left' end
	if oParams.cVertical == nil then oParams.cVertical = 'top' end
	
	-- Figure out its position.
	
	local cTexture = HepHUD.cName .. '/bars.dds'
	local nSize, nPosition, nModify, nX, nY = 51, TOPLEFT, 1, HepHUD.nRight, HepHUD.oBarSize.nHeight / -2
	
	if oParams.cSide == 'left' then nPosition, nModify, nX = TOPRIGHT, -1, HepHUD.nLeft end
	
	if oParams.cVertical == 'bottom' then
		nY, nPosition = nY * -1, BOTTOMLEFT
		if oParams.cSide == 'left' then nPosition = BOTTOMRIGHT end
	end
	
	nX = (HepHUD.nGap + nX * HepHUD.nSpacing) * nModify
	
	-- Create the frames, then the controls inside of them.
	
	local oReturn = chain(WINDOW_MANAGER:CreateTopLevelWindow(nil))
		:SetDimensions(nSize * 5, nSize)
		:SetAnchor(nPosition, GuiRoot, CENTER, nX, nY)
		:SetMouseEnabled(false)
	.__END
	
	-- The buffs/debuffs.
	
	oReturn.aBuffs = {}
	
	local aSet, aTexture = {'power', 'health', 'magicka', 'stamina', 'armor'}
	
	for i = 1, #aSet do
		
		aTexture = HepHUD.getIconCoords(aSet[i])
		
		oReturn.aBuffs[i] = chain(WINDOW_MANAGER:CreateControl(nil, oReturn, CT_TEXTURE))
			:SetMouseEnabled(false)
			:SetTexture(cTexture)
			:SetTextureCoords(aTexture[1], aTexture[1] + 0.1, aTexture[2], aTexture[2] + 0.1)
			:SetDimensions(nSize, nSize)
			:SetColor(1, 1, 1, 1)
			:SetAnchor(nPosition, oReturn, nPosition, nModify * nSize * (i - 1), 0)
			:SetAlpha(0.75)
			:SetHidden(true)
		.__END
		
		oReturn.aBuffs[i].cItem = aSet[i]
		
	end
	
	return oReturn
	
end

function HepHUD.getIconCoords(cKey)
	
	-- Find the coordinates for the passed key.
	
	if cKey == nil or HepHUD.aIcons[cKey] == nil then return end
	
	return HepHUD.aIcons[cKey]
	
end

function HepHUD.toggleVisibility(nEvent, lHidden)
	
	-- Toggles the controls' visibility. Used mainly to turn off the controls
	-- when menus are open. Check to make sure it's a good target, since I
	-- don't want to show the target health if I don't have one no matter what.
	
	local lTarget = HepHUD.isGoodTarget()
	
	HepHUD.lHidden = lHidden
	
	HepHUD.oPlayer:SetHidden(lHidden)
	HepHUD.oMagicka:SetHidden(lHidden)
	HepHUD.oStamina:SetHidden(lHidden)
	HepHUD.oPlayerInfo:SetHidden(lHidden)
	HepHUD.oPlayerBuffs:SetHidden(lHidden)
	HepHUD.oPlayerDebuffs:SetHidden(lHidden)
	HepHUD.oTarget:SetHidden(lHidden or not lTarget)
	HepHUD.oTargetInfo:SetHidden(lHidden or not lTarget)
	HepHUD.oTargetBuffs:SetHidden(lHidden or not lTarget)
	HepHUD.oTargetDebuffs:SetHidden(lHidden or not lTarget)
	
end

function HepHUD.updateBar(oStatus, nVal, nMax)
	
	-- Updates a single bar. Takes in the bar, it's current value,
	-- and the maximum value.
	
	if nVal == nil or nVal < 0 then nVal = 0 end
	if nMax == nil or nMax <= 0 then nMax = nVal end
	if nMax == 0 then nMax = 1 end
	
	-- Calculate the size of the texture. Not only does its size change,
	-- but you have to also change the proportional size.
	
	local nTexture = 1 - nVal / nMax
	local nHeight = HepHUD.oBarSize.nHeight * (nVal / nMax)
	local nAlpha = HepHUD.oAlpha.nNormal
	
	if nVal < nMax then nAlpha = HepHUD.oAlpha.nUsed end
	
	-- Do the settings. Change the alpha dependent on whether or not the
	-- bar is full.
	
	oStatus.oBar:SetTextureCoords(oStatus.aTextureCoords[1], oStatus.aTextureCoords[2], nTexture, 1)
	oStatus.oBar:SetDimensions(HepHUD.oBarSize.nWidth, nHeight)
	oStatus.oBG:SetAlpha(nAlpha)
	oStatus.oBar:SetAlpha(nAlpha)
	
end

function HepHUD.getDifficulty(cUnit)
	
	-- Check the difficulty of the passed unit.
	
	local nDifficulty = GetUnitDifficulty(cUnit)
	local aDifficulties = {[MONSTER_DIFFICULTY_NORMAL] = 'normal', [MONSTER_DIFFICULTY_HARD] = 'hard', [MONSTER_DIFFICULTY_DEADLY] = 'deadly'}
	
	if nDifficulty == nil or aDifficulties[nDifficulty] == nil then return '' end
	
	return aDifficulties[nDifficulty]
	
end

function HepHUD.updateLevel(cUnit, nLevel)
	
	-- Update the rank/level info. Set which info box I'm updating.
	
	local oInfo, nLevel, nRank, nAlliance, lVeteran, lRank, aCoords, aAlliances, cDifficulty, nColor1, nColor2, nColor3
	
	if cUnit == nil then return end
	
	if cUnit == 'player' then
		oInfo = HepHUD.oPlayerInfo
	elseif cUnit == 'reticleover' or cUnit == 'reticleoverplayer' then
		oInfo = HepHUD.oTargetInfo
		cUnit = 'reticleover'
	end
	
	if oInfo == nil then return end
	
	-- Update the level.
	
	nLevel = GetUnitVeteranRank(cUnit)
	
	if nLevel ~= nil and nLevel > 0 then
		lVeteran = true
	else
		nLevel = GetUnitLevel(cUnit)
		lVeteran = false
	end
	
	oInfo.oLevelBG:SetHidden(not lVeteran)
	oInfo.oLevel:SetText(nLevel)
	
	-- Now the rank.
	
	nAlliance = GetUnitAlliance(cUnit)
	aAlliances = {
		[ALLIANCE_ALDMERI_DOMINION] = {'dominion', 185, 147, 59},
		[ALLIANCE_DAGGERFALL_COVENANT] = {'covenant', 65, 110, 151},
		[ALLIANCE_EBONHEART_PACT] = {'ebonheart', 164, 41, 39}}
	
	if nAlliance ~= nil and nAlliance > 0 then
		nRank = GetUnitAvARank(cUnit)
		aCoords = HepHUD.getIconCoords(aAlliances[nAlliance][1])
		nColor1, nColor2, nColor3 = aAlliances[nAlliance][2], aAlliances[nAlliance][3], aAlliances[nAlliance][4]
		lRank = true
	else
		
		-- Check its difficulty.
		
		cDifficulty = HepHUD.getDifficulty(cUnit)
		
		if cDifficulty == '' then
			lRank = false
		else
			aCoords = HepHUD.getIconCoords(cDifficulty)
			nColor1, nColor2, nColor3 = 128, 128, 128
			lRank = true
		end
		
	end
	
	oInfo.oRankBG:SetHidden(not lRank)
	oInfo.oRank:SetHidden(not lRank or nRank == nil or nRank <= 0)
	
	if lRank then
		oInfo.oRankBG:SetTextureCoords(aCoords[1], aCoords[1] + 0.1, aCoords[2], aCoords[2] + 0.1)
		oInfo.oRankBG:SetColor(nColor1 / 255, nColor2 / 255, nColor3 / 255, 1)
		oInfo.oRank:SetText(nRank)
	end
	
end

function HepHUD.defaultsPlayer()
	
	-- Set up the basic player bars.
	
	local aData = {GetUnitPower('player', POWERTYPE_HEALTH)}
	
	if aData == nil then return end
	
	HepHUD.updateBar(HepHUD.oPlayer, aData[1], aData[3])
	
	aData = {GetUnitPower('player', POWERTYPE_MAGICKA)}
	
	HepHUD.updateBar(HepHUD.oMagicka, aData[1], aData[3])
	
	aData = {GetUnitPower('player', POWERTYPE_STAMINA)}
	
	HepHUD.updateBar(HepHUD.oStamina, aData[1], aData[3])
	
	-- Get the player's level and rank info.
	
	HepHUD.updateLevel('player')
	
end



function HepHUD.defaultsTarget()
	
	-- Set up the target bar. Used when starting and triggered when
	-- a target is acquired or lost.
	
	local lShow = HepHUD.isGoodTarget()
	
	if lShow then
		local aData = {GetUnitPower('reticleover', POWERTYPE_HEALTH)}
		HepHUD.updateBar(HepHUD.oTarget, aData[1], aData[3])
		HepHUD.updateLevel('reticleover')
		HepHUD.refreshVisuals(0, 'reticleover')
	end
	
	HepHUD.oTarget:SetHidden(HepHUD.lHidden or not lShow)
	HepHUD.oTargetInfo:SetHidden(HepHUD.lHidden or not lShow)
	HepHUD.oTargetBuffs:SetHidden(HepHUD.lHidden or not lShow)
	HepHUD.oTargetDebuffs:SetHidden(HepHUD.lHidden or not lShow)
	
end

function HepHUD.isGoodTarget()
	
	-- Checks to see if it's a good target. Used FTC for framework.
	-- If the target is blank, don't use it. If the target isn't level
	-- 1, it's not a critter, so use it.
	
	local cTarget = GetUnitName('reticleover')
	
	if cTarget == nil or cTarget == '' then return false end
	if GetUnitLevel('reticleover') ~= 1 then return true end
	
	-- Now the critters. Go through a list of them, not counting it if
	-- I run by any of them.
	
	local aCritters = {'Butterfly', 'Lizard', 'Rat', 'Snake', 'Pony Guar',
		'Frog', 'Squirrel', 'Rabbit', 'Deer', 'Cat', 'Pig', 'Sheep', 'Antelope',
		'Wasp', 'Monkey', 'Fleshflies', 'Centipede', 'Chicken', 'Torchbug',
		'Spider', 'Scorpion', 'Goat', 'Scrib', 'Scuttler'}
	
	for i = 1, #aCritters do
		if cTarget == aCritters[i] then return false end
	end
	
	-- I didn't find one, so it's good.
	
	return true
	
end

function HepHUD.powerHandler(nEvent, cUnit, nIndex, nType, nValue, nMax, nEffective)
	
	-- Handles all power changes. If the unit isn't the player, or it's not the target's
	-- health, don't bother.
	
	if cUnit == nil or (cUnit ~= 'player' and cUnit ~= 'reticleover') then return end
	
	-- Figure out which bar I'm changing.
	
	local oUnit
	
	if cUnit == 'reticleover' and nType == POWERTYPE_HEALTH then
		oUnit = HepHUD.oTarget
	elseif nType == POWERTYPE_HEALTH then
		oUnit = HepHUD.oPlayer
	elseif nType == POWERTYPE_MAGICKA then
		oUnit = HepHUD.oMagicka
	elseif nType == POWERTYPE_STAMINA then
		oUnit = HepHUD.oStamina
	end
	
	-- If I found one, update that bar.
	
	if oUnit ~= nil then
		HepHUD.updateBar(oUnit, nValue, nEffective)
	end
	
end

function HepHUD.checkForRegen(cUnit, nVisual, nStat, lOff)
	
	-- Checks to see if I need to recolor a bar's outline to designate regen/degen.
	-- Start by checking parameters.
	
	if cUnit == nil or (cUnit ~= 'player' and cUnit ~= 'reticleover' and cUnit ~= 'reticleoverplayer') then return end
	if nVisual == nil or nVisual == 0 or nStat == nil or nStat == 0 then return end
	if lOff == nil then lOff = false end
	
	-- Find out if it's a regen or a degen.
	
	local lPlayer = cUnit == 'player'
	local lRegen = nVisual == ATTRIBUTE_VISUAL_INCREASED_REGEN_POWER
	local lDegen = nVisual == ATTRIBUTE_VISUAL_DECREASED_REGEN_POWER
	local oStat
	
	if not lRegen and not lDegen then return end
	
	-- Find out which bar I'm modifying.
	
	if nStat == STAT_HEALTH_REGEN_COMBAT and lPlayer then
		oStat = HepHUD.oPlayer
	elseif nStat == STAT_HEALTH_REGEN_COMBAT then
		oStat = HepHUD.oTarget
	elseif nStat == STAT_MAGICKA_REGEN_COMBAT then
		oStat = HepHUD.oMagicka
	elseif nStat == STAT_STAMINA_REGEN_COMBAT then
		oStat = HepHUD.oStamina
	end
	
	if oStat == nil then return end
	
	-- If it's already showing the regen/degen, leave.
	
	if (lRegen and oStat.lRegen ~= lOff) or (lDegen and oStat.lDegen ~= lOff) then return end
	
	-- Assign them for storage.
	
	if lRegen then oStat.lRegen = not lOff end
	if lDegen then oStat.lDegen = not lOff end
	
	-- Get the colors.
	
	local nTop1, nTop2, nTop3, nBot1, nBot2, nBot3
	local nRegen1, nRegen2, nRegen3, nDegen1, nDegen2, nDegen3 = 255, 255, 255, 0, 0, 0
	local nAlpha = HepHUD.oAlpha.nUsed
	
	if lOff then nAlpha = oStat.oBG:GetAlpha() end
	if lRegen and lOff then nRegen1, nRegen2, nRegen3 = unpack(oStat.aColors) end
	if lDegen and lOff then nDegen1, nDegen2, nDegen3 = unpack(oStat.aColors) end
	
	if lRegen then
		
		nTop1, nTop2, nTop3 = nRegen1, nRegen2, nRegen3
		nBot1, nBot2, nBot3 = nRegen1, nRegen2, nRegen3
		
		if oStat.lDegen then
			nBot1, nBot2, nBot3 = nDegen1, nDegen2, nDegen3
			if lOff then nTop1, nTop2, nTop3 = nDegen1, nDegen2, nDegen3 end
		end
		
	else
		
		nTop1, nTop2, nTop3 = nDegen1, nDegen2, nDegen3
		nBot1, nBot2, nBot3 = nDegen1, nDegen2, nDegen3
		
		if oStat.lRegen then
			nTop1, nTop2, nTop3 = nRegen1, nRegen2, nRegen3
			if lOff then nBot1, nBot2, nBot3 = nRegen1, nRegen2, nRegen3 end
		end
		
	end
	
	-- Finally, assign them.
	
	oStat.oBG:SetGradientColors(ORIENTATION_VERTICAL,
		nTop1 / 255, nTop2 / 255, nTop3 / 255, nAlpha,
		nBot1 / 255, nBot2 / 255, nBot3 / 255, nAlpha)
	
end

function HepHUD.resetVisuals(oBar)
	
	-- Just reset the visuals for a passed bar.
	
	if oBar == nil then return end
	
	oBar.oBG:SetGradientColors(ORIENTATION_VERTICAL,
		oBar.aColors[1] / 255, oBar.aColors[2] / 255, oBar.aColors[3] / 255, HepHUD.oAlpha.nNormal,
		oBar.aColors[1] / 255, oBar.aColors[2] / 255, oBar.aColors[3] / 255, HepHUD.oAlpha.nNormal)
	
	oBar.lRegen = false
	oBar.lDegen = false
	
end

function HepHUD.refreshVisuals(nEvent, cUnit, nVisual, nStat, nAttribute, nPower, nValue, nMax)
	
	-- Because I'm not sure how visuals are updated, and if it's separate or summed (ie, does
	-- having both a health regen of +4 and a health degen of -3 just come back with a health
	-- regen of +1, or do both affects show?), I just refresh the whole thing on an update.
	
	if cUnit == nil then return end
	if cUnit == 'reticleoverplayer' then cUnit = 'reticleover' end
	if cUnit ~= 'player' and cUnit ~= 'reticleover' then return end
	
	-- Reset the bars.
	
	if cUnit ~= 'player' then
		HepHUD.resetVisuals(HepHUD.oTarget)
		HepHUD.resetBuffs(HepHUD.oTargetBuffs)
		HepHUD.resetBuffs(HepHUD.oTargetDebuffs)
	else
		HepHUD.resetVisuals(HepHUD.oHealth)
		HepHUD.resetVisuals(HepHUD.oMagicka)
		HepHUD.resetVisuals(HepHUD.oStamina)
		HepHUD.resetBuffs(HepHUD.oPlayerBuffs)
		HepHUD.resetBuffs(HepHUD.oPlayerDebuffs)
	end
	
	local aVisuals, nTotal, i = {GetAllUnitAttributeVisualizerEffectInfo(cUnit)}
	
	if aVisuals ~= nil and table.getn(aVisuals) > 0 then
		
		-- I have some visuals. cut up the table and iterate it.
		
		local nTotal = table.getn(aVisuals) / 6
		
		for i = 1, nTotal do
			HepHUD.checkForRegen(cUnit, aVisuals[i], aVisuals[i + 1])
			HepHUD.checkStatusIcons(cUnit, aVisuals[i], aVisuals[i + 1])
		end
		
	end
	
end

function HepHUD.resetBuffs(oFrame)
	
	-- Turn off all of the buffs of a single frame.
	
	local aBuffs = oFrame.aBuffs
	
	for i = 1, #aBuffs do
		aBuffs[i]:SetHidden(true)
	end
	
end

function HepHUD.checkStatusIcons(cUnit, nVisual, nStat, lOff)
	
	if cUnit == nil or (cUnit ~= 'player' and cUnit ~= 'reticleover' and cUnit ~= 'reticleoverplayer') then return end
	if nVisual == nil or nVisual == 0 or nStat == nil or nStat == 0 then return end
	if lOff == nil then lOff = false end
	
	-- Find out which bar I'm using.
	
	local lPlayer = cUnit == 'player'
	local lBuff = (nVisual == ATTRIBUTE_VISUAL_INCREASED_MAX_POWER or nVisual == ATTRIBUTE_VISUAL_INCREASED_STAT)
	local lDebuff = (nVisual == ATTRIBUTE_VISUAL_DECREASED_MAX_POWER or nVisual == ATTRIBUTE_VISUAL_DECREASED_STAT)
	
	if not lBuff and not lDebuff then return end
	
	local oFrame
	
	if lBuff and lPlayer then
		oFrame = HepHUD.oPlayerBuffs
	elseif lBuff then
		oFrame = HepHUD.oTargetBuffs
	elseif lDebuff and lPlayer then
		oFrame = HepHUD.oPlayerDebuffs
	else
		oFrame = HepHUD.oTargetDebuffs
	end
	
	-- Find out which buff I'm modifying.
	
	local aBuffs = oFrame.aBuffs
	local cModify = HepHUD.getBuff(nStat)
	
	if cModify == '' then return end
	
	for i = 1, #aBuffs do
		
		if aBuffs[i].cItem == cModify then
			aBuffs[i]:SetHidden(lOff)
			return
		end
		
	end
	
end

function HepHUD.getBuff(nStat)
	
	-- Returns a single buff icon.
	
	local aBuffs = {
		[STAT_HEALTH_MAX] = 'health',
		[STAT_MAGICKA_MAX] = 'magicka',
		[STAT_STAMINA_MAX] = 'stamina',
		[STAT_ARMOR_RATING] = 'armor',
		[STAT_POWER] = 'power'}
	
	if aBuffs[nStat] ~= nil then return aBuffs[nStat] end
	
	return ''
	
end

function HepHUD.updateVisual(nEvent, cUnit, nVisual, nStat, nAttribute, nPower, nValue, nMax)
	
	-- Updates any visuals on the status bars.
	
	local lOff = nEvent == 131408
	
	-- Check for regen/degen.
	
	HepHUD.checkForRegen(cUnit, nVisual, nStat, lOff)
	
	-- Check for icons.
	
	HepHUD.checkStatusIcons(cUnit, nVisual, nStat, lOff)
	
end

function HepHUD.refreshLevel(cUnit, nLevel)
	
	-- Update the unit's info whenever the event is triggered.
	
	if cUnit == nil then return end
	
	if cUnit == 'player' then
		HepHUD.defaultsPlayer()
	elseif cUnit == 'reticleover' or cUnit == 'reticleoverplayer' then
		HepHUD.defaults.Target()
	end
	
end

function HepHUD.init(nEvent, cAddon)
	
	-- Instantiates the HUD. Create all of the bars.
	
	if cAddon ~= HepHUD.cName then return end
	
	HepHUD.oTarget = createBar({cTrack = 'Player Health', aColors = {192, 0, 192}})
	HepHUD.oPlayer = createBar({cTrack = 'Target Health'})
	HepHUD.oMagicka = createBar({cTrack = 'Player Magicka', cSide = 'right', aColors = {62, 154, 222}})
	HepHUD.oStamina = createBar({cTrack = 'Player Stamina', cSide = 'right', aColors = {102, 204, 51}})
	
	HepHUD.oPlayerInfo = createInfoFrame({oAlign = HepHUD.oTarget})
	HepHUD.oTargetInfo = createInfoFrame({oAlign = HepHUD.oMagicka, cSide = 'right'})
	
	HepHUD.oPlayerBuffs = createBuffFrame()
	HepHUD.oTargetBuffs = createBuffFrame({cSide = 'right'})
	HepHUD.oPlayerDebuffs = createBuffFrame({cVertical = 'bottom'})
	HepHUD.oTargetDebuffs = createBuffFrame({cSide = 'right', cVertical = 'bottom'})
	
	-- Fill the defaults.
	
	HepHUD.defaultsPlayer()
	HepHUD.defaultsTarget()
	
	-- Register events.
	
	EVENT_MANAGER:RegisterForEvent('HepHUD', EVENT_RETICLE_TARGET_CHANGED, HepHUD.defaultsTarget)
	EVENT_MANAGER:RegisterForEvent('HepHUD', EVENT_PLAYER_ALIVE, HepHUD.defaultsPlayer)
	EVENT_MANAGER:RegisterForEvent('HepHUD', EVENT_POWER_UPDATE, HepHUD.powerHandler)
	EVENT_MANAGER:RegisterForEvent('HepHUD', EVENT_RETICLE_HIDDEN_UPDATE, HepHUD.toggleVisibility)
	EVENT_MANAGER:RegisterForEvent('HepHUD', EVENT_LEVEL_UPDATE, HepHUD.refreshLevel)
	EVENT_MANAGER:RegisterForEvent('HepHUD', EVENT_UNIT_ATTRIBUTE_VISUAL_ADDED, HepHUD.updateVisual)
	EVENT_MANAGER:RegisterForEvent('HepHUD', EVENT_UNIT_ATTRIBUTE_VISUAL_REMOVED, HepHUD.updateVisual)
	EVENT_MANAGER:RegisterForEvent('HepHUD', EVENT_UNIT_ATTRIBUTE_VISUAL_UPDATED, HepHUD.updateVisual)
	
end

EVENT_MANAGER:RegisterForEvent('HepHUD', EVENT_ADD_ON_LOADED, HepHUD.init)